@@ -643,16 +643,17 @@ myorigin(struct header *hp)
    return rv;
 }
 
-FL int
-is_head(char const *linebuf, size_t linelen, bool_t compat)
+FL bool_t
+is_head(char const *linebuf, size_t linelen, bool_t check_rfc4155)
 {
    char date[FROM_DATEBUF];
-   int rv;
+   bool_t rv;
    NYD2_ENTER;
 
-   if ((rv = (linelen >= 5 && !strncmp(linebuf, "From ", 5))) &&
-         (!compat || ok_blook(mbox_rfc4155)))
-      rv = (extract_date_from_from_(linebuf, linelen, date) && _is_date(date));
+   if ((rv = (linelen >= 5 && !memcmp(linebuf, "From ", 5))) && check_rfc4155 &&
+         (extract_date_from_from_(linebuf, linelen, date) <= 0 ||
+          !_is_date(date)))
+      rv = TRUM1;
    NYD2_LEAVE;
    return rv;
 }
@@ -661,10 +662,12 @@ FL int
 extract_date_from_from_(char const *line, size_t linelen,
    char datebuf[FROM_DATEBUF])
 {
-   int rv = 0;
+   int rv;
    char const *cp = line;
    NYD_ENTER;
 
+   rv = 1;
+
    /* "From " */
    cp = _from__skipword(cp);
    if (cp == NULL)
@@ -678,6 +681,23 @@ extract_date_from_from_(char const *line, size_t linelen,
       if (cp == NULL)
          goto jerr;
    }
+   /* It seems there are invalid MBOX archives in the wild, compare
+    * . http://bugs.debian.org/624111
+    * . [Mutt] #3868: mutt should error if the imported mailbox is invalid
+    * What they do is that they obfuscate the address to "name at host",
+    * and even "name at host dot dom dot dom.  I think we should handle that */
+   else if(cp[0] == 'a' && cp[1] == 't' && cp[2] == ' '){
+      rv = -1;
+      cp += 3;
+jat_dot:
+      cp = _from__skipword(cp);
+      if (cp == NULL)
+         goto jerr;
+      if(cp[0] == 'd' && cp[1] == 'o' && cp[2] == 't' && cp[3] == ' '){
+         cp += 4;
+         goto jat_dot;
+      }
+   }
 
    linelen -= PTR2SIZE(cp - line);
    if (linelen < _DATE_MINLEN)
@@ -693,7 +713,6 @@ extract_date_from_from_(char const *line, size_t linelen,
    if (linelen >= FROM_DATEBUF)
       goto jerr;
 
-   rv = 1;
 jleave:
    memcpy(datebuf, cp, linelen);
    datebuf[linelen] = '\0';
@@ -704,6 +723,7 @@ jerr:
    linelen = strlen(cp);
    if (linelen >= FROM_DATEBUF)
       linelen = FROM_DATEBUF;
+   rv = 0;
    goto jleave;
 }
 
@@ -1500,7 +1520,7 @@ jnewname:
       goto jout;
    if ((cp = strchr(linebuf, 'F')) == NULL)
       goto jout;
-   if (strncmp(cp, "From", 4)) /* XXX is_head? */
+   if (strncmp(cp, "From", 4))
       goto jout;
    if (namesize <= linesize)
       namebuf = srealloc(namebuf, namesize = linesize + 1);
@@ -1601,6 +1621,11 @@ msgidcmp(char const *s1, char const *s2)
    int q1 = 0, q2 = 0, c1, c2;
    NYD_ENTER;
 
+   while(*s1 == '<')
+      ++s1;
+   while(*s2 == '<')
+      ++s2;
+
    do {
       c1 = msgidnextc(&s1, &q1);
       c2 = msgidnextc(&s2, &q2);
