@@ -168,8 +168,6 @@ jdocopy:
    /* Don't display an absolute path but "+FOLDER" if under *folder* */
    if (getfold(tbuf, sizeof tbuf)) {
       i = strlen(tbuf);
-      if (i < sizeof(tbuf) -1)
-         tbuf[i++] = '/';
       if (!strncmp(tbuf, mailp, i)) {
          mailp += i;
          *dispp++ = '+';
@@ -482,6 +480,9 @@ setfile(char const *name, enum fedit_mode fm) /* TODO oh my god */
    if ((name = expand(name)) == NULL)
       goto jem1;
 
+   /* For at least substdate() users */
+   time_current_update(&time_current, FAL0);
+
    switch (which_protocol(name)) {
    case PROTO_FILE:
       if (temporary_protocol_ext != NULL)
@@ -862,7 +863,7 @@ jreadline:
             goto jreadline;
          /* That *can* happen since evaluate() unstack()s on error! */
          if (temporary_orig_line != NULL)
-            tty_addhist(temporary_orig_line, !ev.ev_add_history);
+            tty_addhist(temporary_orig_line, (ev.ev_add_history != TRU1));
       }
    }
 
@@ -1112,8 +1113,9 @@ je96:
       evp->ev_new_content = cp;
       goto jleave0;
    }
-   if (!(com->argtype & ARG_H) && !(pstate & PS_MSGLIST_SAW_NO))
-      evp->ev_add_history = TRU1;
+   if (!(com->argtype & ARG_H))
+      evp->ev_add_history = (((com->argtype & ARG_G) ||
+            (pstate & PS_MSGLIST_SAW_NO)) ? TRUM1 : TRU1);
 
 jleave:
    /* Exit the current source file on error TODO what a mess! */
@@ -1425,6 +1427,7 @@ initbox(char const *name)
    Ftmp_release(&tempMesg);
 
    message_reset();
+   mb.mb_active = MB_NONE;
    mb.mb_threaded = 0;
    if (mb.mb_sorted != NULL) {
       free(mb.mb_sorted);
