@@ -148,8 +148,9 @@ do {\
 #else
 # define really_rewind(stream) \
 do {\
-   fseek(stream, 0, SEEK_END);\
    rewind(stream);\
+   fflush(stream);\
+   lseek(fileno(stream), 0, SEEK_SET);\
 } while (0)
 #endif
 
@@ -919,13 +920,16 @@ FL char const * myaddrs(struct header *hp);
 FL char const * myorigin(struct header *hp);
 
 /* See if the passed line buffer, which may include trailing newline (sequence)
- * is a mail From_ header line according to RFC 4155.
- * If compat is true laxe POSIX syntax is instead sufficient to match From_ */
-FL int         is_head(char const *linebuf, size_t linelen, bool_t compat);
+ * is a mail From_ header line according to POSIX ("From ").
+ * If check_rfc4155 is true we'll return TRUM1 instead if the From_ line
+ * matches POSIX but is _not_ compatible to RFC 4155 */
+FL bool_t      is_head(char const *linebuf, size_t linelen,
+                  bool_t check_rfc4155);
 
 /* Savage extract date field from From_ line.  linelen is convenience as line
  * must be terminated (but it may end in a newline [sequence]).
- * Return wether the From_ line was parsed successfully */
+ * Return wether the From_ line was parsed successfully (-1 if the From_ line
+ * wasn't really RFC 4155 compliant) */
 FL int         extract_date_from_from_(char const *line, size_t linelen,
                   char datebuf[FROM_DATEBUF]);
 
@@ -1053,6 +1057,10 @@ FL bool_t      header_match(struct message *mp, struct search_expr const *sep);
  */
 
 #ifdef HAVE_IMAP
+/* The former returns the input again if no conversion is necessary */
+FL char const *imap_path_encode(char const *path, bool_t *err_or_null);
+FL char *      imap_path_decode(char const *path, bool_t *err_or_null);
+
 FL char const * imap_fileof(char const *xcp);
 FL enum okay   imap_noop(void);
 FL enum okay   imap_select(struct mailbox *mp, off_t *size, int *count,
@@ -1064,6 +1072,7 @@ FL void        imap_getheaders(int bot, int top);
 FL void        imap_quit(void);
 FL enum okay   imap_undelete(struct message *m, int n);
 FL enum okay   imap_unread(struct message *m, int n);
+FL int         c_imapcodec(void *v);
 FL int         c_imap_imap(void *vp);
 FL int         imap_newmail(int nmail);
 FL enum okay   imap_append(const char *xserver, FILE *fp);
@@ -1085,6 +1094,7 @@ FL void        transflags(struct message *omessage, long omsgCount,
 FL time_t      imap_read_date_time(const char *cp);
 FL const char * imap_make_date_time(time_t t);
 #else
+# define c_imapcodec             c_cmdnotsupp
 # define c_imap_imap             c_cmdnotsupp
 # define c_connect               c_cmdnotsupp
 # define c_disconnect            c_cmdnotsupp
@@ -1425,7 +1435,7 @@ FL char const * mime_type_mimepart_handler(struct mimepart const *mpp);
 
 /* Allocate a single element of a name list, initialize its name field to the
  * passed name and return it */
-FL struct name * nalloc(char *str, enum gfield ntype);
+FL struct name * nalloc(char const *str, enum gfield ntype);
 
 /* Like nalloc(), but initialize from content of np */
 FL struct name * ndup(struct name *np, enum gfield ntype);
@@ -1944,12 +1954,12 @@ FL struct str * n_str_add_buf(struct str *self, char const *buf, size_t buflen
 
 /* ..and update arguments to point after range; returns UI32_MAX on error, in
  * which case the arguments will have been stepped one byte */
-#ifdef HAVE_NATCH_CHAR
+#if defined HAVE_NATCH_CHAR || defined HAVE_IMAP
 FL ui32_t      n_utf8_to_utf32(char const **bdat, size_t *blen);
 #endif
 
 /* buf must be large enough also for NUL, it's new length will be returned */
-#ifdef HAVE_FILTER_HTML_TAGSOUP
+#if defined HAVE_FILTER_HTML_TAGSOUP || defined HAVE_IMAP
 FL size_t      n_utf32_to_utf8(ui32_t c, char *buf);
 #endif
 
@@ -2077,6 +2087,13 @@ FL char *      urlxdec(char const *cp SALLOC_DEBUG_ARGS);
 # define urlxdec(CP)             urlxdec(CP, __FILE__, __LINE__)
 #endif
 
+/* `urlcodec' */
+FL int         c_urlcodec(void *v);
+
+/* Parse a RFC 6058 'mailto' URI to a single to: (TODO yes, for now hacky).
+ * Return NULL or something that can be converted to a struct name */
+FL char *      url_mailto_to_address(char const *mailtop);
+
 #ifdef HAVE_SOCKETS
 /* Return port of urlp->url_proto (and set irv_or_null), or NULL if unknown */
 FL char const * url_servbyname(struct url const *urlp, ui16_t *irv_or_null);
