@@ -324,10 +324,10 @@ ifdef HAVE_DEBUG /* TODO assert legacy */
     * TODO if it may happen in this path, we should just treat decryption
     * TODO as we do for the other input paths; i.e., handle it in SSL!! */
    if (action == SEND_MBOX || action == SEND_DECRYPT)
-      assert(!is_head(buf, len, TRU1));
+      assert(!is_head(buf, len, FAL0));
 #else
    if ((/*action == SEND_MBOX ||*/ action == SEND_DECRYPT) &&
-         is_head(buf, len, TRU1)) {
+         is_head(buf, len, FAL0)) {
       putc('>', fp);
       ++sz;
    }
@@ -1084,6 +1084,8 @@ newfile(struct mimepart *ip, int *ispipe)
       free(out.s);
    }
 
+   /* In interactive mode, let user perform all kind of expansions as desired,
+    * and offer |SHELL-SPEC pipe targets, too */
    if (options & OPT_INTERACTIVE) {
       struct str prompt;
       char *f2, *f3;
@@ -1112,22 +1114,52 @@ jgetname:
       else
          f = f3;
    }
-   if (f == NULL || f == (char*)-1) {
-      fp = NULL;
-      goto jleave;
-   }
 
-   if (*f == '|') {
-      char const *cp;
-      cp = ok_vlook(SHELL);
-      if (cp == NULL)
-         cp = XSHELL;
-      fp = Popen(f + 1, "w", cp, NULL, 1);
-      if (!(*ispipe = (fp != NULL)))
-         n_perr(f, 0);
-   } else {
-      if ((fp = Fopen(f, "w")) == NULL)
+   if (f == NULL || f == (char*)-1 || *f == '\0')
+      fp = NULL;
+   else if (options & OPT_INTERACTIVE) {
+      if (*f == '|') {
+         char const *cp;
+         cp = ok_vlook(SHELL);
+         if (cp == NULL)
+            cp = XSHELL;
+         fp = Popen(f + 1, "w", cp, NULL, 1);
+         if (!(*ispipe = (fp != NULL)))
+            n_perr(f, 0);
+      } else if ((fp = Fopen(f, "w")) == NULL)
          n_err(_("Cannot open \"%s\"\n"), f);
+   } else {
+      /* Be very picky in non-interactive mode: actively disallow pipes,
+       * prevent directory separators, and any filename member that would
+       * become expanded by the shell if the name would be echo(1)ed */
+      if(anyof(f, "/" n_SHEXP_MAGIC_PATH_CHARS)){
+         char c;
+
+         for(out.s = salloc((strlen(f) * 3) +1), out.l = 0; (c = *f++) != '\0';)
+            if(strchr("/" n_SHEXP_MAGIC_PATH_CHARS, c)){
+               out.s[out.l++] = '%';
+               mime_char_to_hexseq(&out.s[out.l], c);
+               out.l += 2;
+            }else
+               out.s[out.l++] = c;
+         out.s[out.l] = '\0';
+         f = out.s;
+      }
+
+      /* Avoid overwriting of existing files */
+      while((fp = Fopen(f, "wx")) == NULL){
+         int e;
+
+         if((e = errno) != EEXIST){
+            n_err(_("Cannot open \"%s\": %s\n"), f, strerror(e));
+            break;
+         }
+
+         if(ip->m_partstring != NULL)
+            f = savecatsep(f, '#', ip->m_partstring);
+         else
+            f = savecat(f, "#.");
+      }
    }
 jleave:
    NYD_LEAVE;
@@ -1247,6 +1279,8 @@ sendmp(struct message *mp, FILE *obuf, struct ignoretab *doign,
    int rv = -1, c;
    NYD_ENTER;
 
+   time_current_update(&time_current, TRU1);
+
    if (mp == dot && action != SEND_TOSRCH)
       pstate |= PS_DID_PRINT_DOT;
    if (stats != NULL)
@@ -1274,34 +1308,30 @@ sendmp(struct message *mp, FILE *obuf, struct ignoretab *doign,
                (int)qf.qf_pfix_len, (qf.qf_pfix_len != 0 ? qf.qf_pfix : ""),
                fakefrom(mp), fakedate(mp->m_time),
                (csuf != NULL ? csuf->s : ""));
-   } else {
-      if (doign != allignore && doign != fwdignore && action != SEND_RFC822) {
-         if (qf.qf_pfix_len > 0) {
-            i = fwrite(qf.qf_pfix, sizeof *qf.qf_pfix, qf.qf_pfix_len, obuf);
-            if (i != qf.qf_pfix_len)
-               goto jleave;
-            sz += i;
-         }
-#ifdef HAVE_COLOUR
-         if (cpre != NULL) {
-            fputs(cpre->s, obuf);
-            cpre = (struct str const*)0x1;
-         }
-#endif
+   } else if (doign != allignore && doign != fwdignore &&
+         action != SEND_RFC822) {
+      if (qf.qf_pfix_len > 0) {
+         i = fwrite(qf.qf_pfix, sizeof *qf.qf_pfix, qf.qf_pfix_len, obuf);
+         if (i != qf.qf_pfix_len)
+            goto jleave;
+         sz += i;
       }
+#ifdef HAVE_COLOUR
+      if (cpre != NULL) {
+         fputs(cpre->s, obuf);
+         cpre = (struct str const*)0x1;
+      }
+#endif
 
       while (cnt > 0 && (c = getc(ibuf)) != EOF) {
-         if (doign != allignore && doign != fwdignore &&
-               action != SEND_RFC822) {
 #ifdef HAVE_COLOUR
-            if (c == '\n' && csuf != NULL) {
-               cpre = (struct str const*)0x1;
-               fputs(csuf->s, obuf);
-            }
-#endif
-            putc(c, obuf);
-            sz++;
+         if (c == '\n' && csuf != NULL) {
+            cpre = (struct str const*)0x1;
+            fputs(csuf->s, obuf);
          }
+#endif
+         putc(c, obuf);
+         ++sz;
          --cnt;
          if (c == '\n')
             break;
@@ -1311,6 +1341,12 @@ sendmp(struct message *mp, FILE *obuf, struct ignoretab *doign,
       if (csuf != NULL && cpre != (struct str const*)0x1)
          fputs(csuf->s, obuf);
 #endif
+   } else {
+      while (cnt > 0 && (c = getc(ibuf)) != EOF) {
+         --cnt;
+         if (c == '\n')
+            break;
+      }
    }
    }
    if (sz > 0 && stats != NULL)
