@@ -1043,9 +1043,9 @@ FL struct str *
  * UTF-8
  */
 
-#ifdef HAVE_NATCH_CHAR
+#if defined HAVE_NATCH_CHAR || defined HAVE_IMAP
 FL ui32_t
-n_utf8_to_utf32(char const **bdat, size_t *blen)
+n_utf8_to_utf32(char const **bdat, size_t *blen) /* TODO check false UTF8 */
 {
    char const *cp;
    size_t l;
@@ -1060,12 +1060,12 @@ n_utf8_to_utf32(char const **bdat, size_t *blen)
       c = x;
    else {
       if ((x & 0xE0) == 0xC0) {
-         if (l < 2)
+         if (l < 1)
             goto jerr;
          l -= 1;
          c = x & ~0xC0;
       } else if ((x & 0xF0) == 0xE0) {
-         if (l < 3)
+         if (l < 2)
             goto jerr;
          l -= 2;
          c = x & ~0xE0;
@@ -1073,7 +1073,7 @@ n_utf8_to_utf32(char const **bdat, size_t *blen)
          x = (ui8_t)*cp++;
          c |= x & 0x7F;
       } else {
-         if (l < 4)
+         if (l < 3)
             goto jerr;
          l -= 3;
          c = x & ~0xF0;
@@ -1098,9 +1098,9 @@ jerr:
    c = UI32_MAX;
    goto jleave;
 }
-#endif /* HAVE_NATCH_CHAR */
+#endif /* HAVE_NATCH_CHAR || IMAP */
 
-#ifdef HAVE_FILTER_HTML_TAGSOUP
+#if defined HAVE_FILTER_HTML_TAGSOUP || defined HAVE_IMAP
 FL size_t
 n_utf32_to_utf8(ui32_t c, char *buf)
 {
@@ -1123,7 +1123,7 @@ n_utf32_to_utf8(ui32_t c, char *buf)
        * xxx (However, should we ever get yet another surrogate range we
        * xxx need to deal with that all over the place anyway? */
       {0x00000800, 0x0000FFFF, 0xE0, 3, 0xF0, 0xFF-0xF0, 3-1, 3, {0,}},
-      {0x00010000, 0x001FFFFF, 0xF0, 4, 0xF8, 0xFF-0xF8, 4-1, 4, {0,}},
+      {0x00010000, 0x0010FFFF, 0xF0, 4, 0xF8, 0xFF-0xF8, 4-1, 4, {0,}},
    }, *catp = _cat;
    size_t l;
 
@@ -1156,7 +1156,7 @@ j0:
    NYD2_LEAVE;
    return l;
 }
-#endif /* HAVE_FILTER_HTML_TAGSOUP */
+#endif /* HAVE_FILTER_HTML_TAGSOUP || IMAP */
 
 /*
  * Our iconv(3) wrapper
